@@ -31,7 +31,7 @@ st_dimensions.dimensions = function(.x, ...) .x
 `st_dimensions<-.stars_proxy` = function(x, value) {
 	if (!is.null(attr(x, "call_list")))
 		stop("st_dimensions<- on a stars_proxy object only works if there is no call list")
-	NextMethod()
+	structure(NextMethod(), class = class(x))
 }
 
 
@@ -95,7 +95,7 @@ st_dimensions.default = function(.x, ..., .raster, affine = c(0, 0),
 
 #' @name st_dimensions
 #' @param which integer or character; index or name of the dimension to be changed
-#' @param values values for this dimension (e.g. \code{sfc} list-column), or length-1 \code{dimensions} object
+#' @param values values for this dimension (e.g. \code{sfc} list-column), or length-1 \code{dimensions} object; setting special value \code{NULL} removes dimension values, for instance to remove curvilinear raster coordinates
 #' @param names character; vector with new names for all dimensions, or with the single new name for the dimension indicated by \code{which}
 #' @param xy length-2 character vector; (new) names for the \code{x} and \code{y} raster dimensions
 #' @export
@@ -113,39 +113,57 @@ st_dimensions.default = function(.x, ..., .raster, affine = c(0, 0),
 #' # set bandwidth intervals:
 #' (x3 = st_set_dimensions(x, "band", values = make_intervals(bw), names = "bandwidth"))
 st_set_dimensions = function(.x, which, values = NULL, point = NULL, names = NULL, xy, ...) {
+	if (inherits(.x, "mdim"))
+		stop("for mdim objects, use st_set_dimensions() after st_as_stars()")
 	d = st_dimensions(.x)
 	if (!missing(which) && is.character(which))
 		which = match(which, base::names(d))
-	if (! is.null(values)) {
+	if (! missing(values)) {
 		if (is.na(which))
 			stop("which should be a name or index of an existing dimensions")
+		if (inherits(values, "units") && names(d)[which] %in% attr(d, "raster")$xy 
+				&& !is.na(st_crs(.x)))
+			stop("units in x/y dimension values only allowed if object has no CRS")
 		if (inherits(values, "dimensions")) {
 			if (is.null(names))
 				names <- names(values)
 			values <- values[[1]]$values
 		}
-		if (!inherits(values, "intervals") && dim(.x)[which] != length(values)) {
+		if (!is.null(values) && !inherits(values, "intervals") && dim(.x)[which] != length(values)) {
 			if (dim(.x)[which] == length(values) - 1) # create intervals:
 				values = as_intervals(values)
 			else
 				stop(paste("length of values (", length(values), 
 					") does not match length of dimension", which, "(", dim(.x)[which], ")"))
 		}
-		d[[which]] = create_dimension(values = values, point = point %||% d[[which]]$point, ...)
-		if (! is.null(names) && length(names) == 1)
-			base::names(d)[which] = names
+		if (is.null(values))
+			d[[which]]["values"] = list(NULL) # avoid removing element values
+		else
+			d[[which]] = create_dimension(values = values, point = point %||% d[[which]]$point, ...)
 		r = attr(d, "raster")
 		if (isTRUE(r$curvilinear)) {
 			# FIXME: there's much more that should be checked for curvilinear grids...
 			# https://github.com/r-spatial/stars/issues/460
-			if (which %in% r$dimensions && !is.matrix(values))
+			if (base::names(d)[which] %in% r$dimensions && !any(sapply(d, function(v) is.matrix(v$values)))) {
 				attr(d, "raster")$curvilinear = FALSE 
+				st_crs(d) = NA
+			}
 		}
 #		else if (inherits(values, "sfc"))
 #			base::names(d)[which] = "sfc"
-	} else if (!is.null(point) && is.logical(point)) {
+	}
+	if (!is.null(point) && is.logical(point)) {
 		d[[which]]$point = point
-	} else if (!missing(names)) {
+	}
+	if (! missing(xy)) {
+		if (is.null(xy) || (length(xy) == 1 && is.na(xy)))
+			xy = c(NA, NA)
+		stopifnot(length(xy) == 2)
+		r = attr(d, "raster")
+		r$dimensions = as.character(xy)
+		attr(d, "raster") = r
+	}
+	if (!missing(names)) {
 		if (length(d) != length(names) && length(names) != 1)
 			stop("length of names should match number of dimensions")
 		r = attr(d, "raster")
@@ -163,13 +181,10 @@ st_set_dimensions = function(.x, which, values = NULL, point = NULL, names = NUL
 		}
 		attr(d, "raster") = r
 		base::names(d) = new_names
-	} else if (! missing(xy)) {
-		stopifnot(length(xy) == 2)
-		r = attr(d, "raster")
-		r$dimensions = as.character(xy)
-		attr(d, "raster") = r
-	} else
+	}
+	if (length(list(...)))
 		d[[which]] = create_dimension(from = 1, to = dim(.x)[which], ...)
+
 	if (inherits(.x, "stars_proxy"))
 		structure(.x, dimensions = d)
 	else
@@ -210,7 +225,7 @@ st_get_dimension_values = function(.x, which, ..., where = NA, max = FALSE, cent
       stop("where, if not NA, must be 'start', 'center' or 'end': ", where)
     }
   }
-	expand_dimensions(.x, ..., max = max, center = center)[[which]]
+  expand_dimensions(.x, ..., max = max, center = center)[[which]]
 }
 
 
@@ -228,7 +243,7 @@ regular_intervals = function(x, epsilon = 1e-10) {
 	if (length(x) <= 1)
 		FALSE
 	else {
-		ud = if (is.atomic(x) && (is.numeric(x) || inherits(x, c("POSIXt", "Date"))))
+		ud = if (is.atomic(x) && (is.numeric(x) || inherits(x, c("POSIXt", "Date", "PCICt"))))
 				unique(diff(x))
 			else {
 				if (inherits(x, "intervals") && identical(tail(x$end, -1), head(x$start, -1)))
@@ -236,7 +251,8 @@ regular_intervals = function(x, epsilon = 1e-10) {
 				else
 					return(FALSE)
 			}
-		isTRUE(as.numeric(abs(diff(range(ud)) / mean(ud))) < epsilon)
+		ud = as.numeric(ud)
+		isTRUE((abs(diff(range(ud)) / mean(ud))) < epsilon)
 	}
 }
 
@@ -275,12 +291,12 @@ create_dimension = function(from = 1, to, offset = NA_real_, delta = NA_real_,
 		else if (inherits(example, "Date"))
 			refsys = "Date"
 		else if (inherits(example, "PCICt"))
-			refsys = "PCICt"
+			refsys = paste0("PCICt_", attr(example, "cal"))
 		else if (inherits(example, "units"))
 			refsys = "udunits"
 
 		if (inherits(values, "sfc")) {
-			point = inherits(values, "sfc_POINT")
+			point = inherits(values, c("sfc_POINT", "sfc_MULTIPOINT"))
 			if (!is.na(st_crs(values)) && is.na(refsys)) # inherit:
 				refsys = st_crs(values)
 		}
@@ -307,8 +323,25 @@ create_dimensions = function(lst, raster = NULL) {
 		names(lst)[sel] = make.names(seq_along(sel))
 	}
 	if (is.null(raster))
-		raster = get_raster(dimensions = c(NA_character_, NA_character_))
-	structure(lst, raster = raster, class = "dimensions")
+		structure(lst, raster = get_raster(dimensions = c(NA_character_, NA_character_)), class = "dimensions")
+	else { 
+		d = structure(lst, raster = raster, class = "dimensions")
+		rd = raster$dimensions
+		if (identical(d[[rd[1]]]$refsys, "udunits") && identical(d[[rd[2]]]$refsys, "udunits")) {
+			deg = units(as_units("degree"))
+			ux = units(d[[rd[1]]])
+			uy = units(d[[rd[2]]])
+			u1 = units(units::as_units("1"))
+			if (inherits(ux, "symbolic_units") && inherits(uy, "symbolic_units") &&
+					(!identical(ux, u1) && !identical(uy, u1)) &&
+					units::ud_are_convertible(ux, deg) && units::ud_are_convertible(uy, deg)) {
+				d[[rd[1]]] = drop_units(d[[rd[1]]])
+				d[[rd[2]]] = drop_units(d[[rd[2]]])
+				d[[rd[1]]]$refsys = d[[rd[2]]]$refsys = st_crs('OGC:CRS84') # specifies units
+			}
+		}
+		d
+	}
 }
 
 get_crs = function(pr) {
@@ -364,31 +397,30 @@ create_dimensions_from_gdal_meta = function(dims, pr) {
 	# handle band descriptions, if present:
 	if (!is.null(lst$band) && !is.null(pr$descriptions) && all(pr$descriptions != ""))
 		lst$band$values = pr$descriptions
+	else if (!is.null(pr$band_meta)) {
+		bm = unlist(pr$band_meta)
+		if (any(a <- grepl("DESCRIPTION=", bm)) && length(which(a)) > 1)
+			lst$band$values = substring(bm[a], 13)
+	}
 	# set up raster:
-	raster = get_raster(affine = pr$geotransform[c(3,5)], dimensions = c("x", "y"), curvilinear = FALSE)
+	raster = get_raster(affine = pr$geotransform[c(3,5)],
+						dimensions = c("x", "y"),
+						curvilinear = FALSE, 
+						blocksizes = pr$blocksizes)
 	create_dimensions(lst, raster)
 }
 
-get_raster = function(affine = rep(0, 2), dimensions = c("x", "y"), curvilinear = FALSE) {
-	if (any(is.na(affine))) {
-		# warning("setting NA affine values to zero")
+get_raster = function(affine = rep(0, 2), dimensions = c("x", "y"),
+					  curvilinear = FALSE, blocksizes = NULL) {
+	if (any(is.na(affine)))
 		affine = c(0, 0)
-	}
-	structure(list(affine = affine, dimensions = dimensions, curvilinear = curvilinear), class = "stars_raster")
-}
-
-get_geotransform = function(x) {
-	if (inherits(x, "stars"))
-		x = st_dimensions(x)
-	stopifnot(inherits(x, "dimensions"))
-	r = attr(x, "raster")
-	if (is.null(r))
-		rep(NA_real_, 6)
-	else {
-		xd = x[[ r$dimensions[1] ]]
-		yd = x[[ r$dimensions[2] ]]
-		c(xd$offset, xd$delta, r$affine[1], yd$offset, r$affine[2], yd$delta)
-	}
+	if (!is.null(blocksizes))
+		colnames(blocksizes) = dimensions # columns, rows!
+	structure(list(affine = affine,
+				   dimensions = dimensions,
+				   curvilinear = curvilinear,
+				   blocksizes = blocksizes), 
+		  class = "stars_raster")
 }
 
 
@@ -397,7 +429,7 @@ print.stars_raster = function(x, ...) {
 	if (any(is.na(x$affine)))
 		cat(paste("affine parameters:", x$affine[1], x$affine[2], "\n"))
 	else if (any(x$affine != 0.0))
-		cat(paste("sheared raster with parameters:", x$affine[1], x$affine[2], "\n"))
+		cat(paste("sheared raster with parameters:", format(x$affine[1], ...), format(x$affine[2], ...), "\n"))
 	if (x$curvilinear)
 		cat("curvilinear grid\n")
 	invisible(x)
@@ -411,6 +443,23 @@ get_val = function(pattern, meta) {
 		NA_character_
 }
 
+get_pcict = function(values, unts, calendar) {
+	stopifnot(calendar %in% c("360_day", "365_day", "noleap"))
+	if (!requireNamespace("PCICt", quietly = TRUE))
+		stop("package PCICt required, please install it first") # nocov
+	origin = 0:1
+	units(origin) = try_as_units(unts)
+	delta = if (grepl("months", unts)) { # for these calendars, length of months are different from udunits representation
+			if (calendar == "360_day")
+				set_units(30 * 24 * 3600, "s", mode = "standard")
+			else
+				set_units((365./12) * 24 * 3600, "s", mode = "standard")
+		} else
+			set_units(as_units(diff(as.POSIXct(origin))), "s", mode = "standard")
+	origin_txt = as.character(as.POSIXct(origin[1]))
+	PCICt::as.PCICt(values * as.numeric(delta), calendar, origin_txt)
+}
+
 parse_netcdf_meta = function(pr, name) {
 	meta = pr$meta
 	spl = strsplit(name, ":")[[1]] # "C:\..." results in length 2:
@@ -439,30 +488,16 @@ parse_netcdf_meta = function(pr, name) {
 					rhs = get_val(paste0("NETCDF_DIM_", v), meta) # nocov # FIXME: find example?
 					pr$dim_extra[[v]] = as.numeric(rhs)           # nocov
 				}
-
 				cal = get_val(paste0(v, "#calendar"), meta)
 				u =   get_val(paste0(v, "#units"), meta)
 				if (! is.na(u)) {
-					if (v == "time" && !is.na(cal) && cal %in% c("360_day", "365_day", "noleap")) {
-						origin = 0:1
-						units(origin) = try_as_units(u)
-						delta = if (grepl("months", u)) {
-								if (cal == "360_day")
-									set_units(30 * 24 * 3600, "s", mode = "standard")
-								else
-									set_units((365/12) * 24 * 3600, "s", mode = "standard")
-							} else
-								set_units(as_units(diff(as.POSIXct(origin))), "s", mode = "standard")
-						origin_txt = as.character(as.POSIXct(origin[1]))
-						if (!requireNamespace("PCICt", quietly = TRUE))
-							stop("package PCICt required, please install it first") # nocov
-						pr$dim_extra[[v]] = PCICt::as.PCICt(pr$dim_extra[[v]] * as.numeric(delta), cal, origin_txt)
-					} else {
+					if (v %in% c("t", "time") && !is.na(cal) && cal %in% c("360_day", "365_day", "noleap"))
+						pr$dim_extra[[v]] = get_pcict(pr$dim_extra[[v]], u, cal)
+					else {
 						units(pr$dim_extra[[v]]) = try_as_units(u)
-						if (v == "time" && !inherits(try(as.POSIXct(pr$dim_extra[[v]]), silent = TRUE),
-								"try-error")) {
+						if (v %in% c("t", "time") && !inherits(try(as.POSIXct(pr$dim_extra[[v]]), silent = TRUE),
+								"try-error"))
 							pr$dim_extra[[v]] = as.POSIXct(pr$dim_extra[[v]])
-						}
 					}
 				}
 			}
@@ -494,18 +529,30 @@ parse_gdal_meta = function(properties) {
 	properties
 }
 
+#' expand the dimension values into a list
+#'
+#' expand the dimension values into a list
+#' @param x object of class `stars` or `dimensions`
+#' @param ... ignored
+#' @export
 expand_dimensions = function(x, ...) UseMethod("expand_dimensions")
 
+#' @export
 expand_dimensions.stars = function(x, ...) {
 	expand_dimensions(st_dimensions(x), ...)
 }
 
+#' @export
+#' @name expand_dimensions
+#' @param max logical; if `TRUE` return the max (end) values of the dimensions intervals
+#' @param center logical; if `TRUE` return the center values of intervals, otherwise return offset (start) of intervals; if `NA` (default) return centers for x/y dimensions, offsets for all others
+expand_dimensions.dimensions = function(x, ..., max = FALSE, center = NA) {
 # returns, in case of numeric dimensions:
 # 	center = TRUE: return center values for x and y coordinates, interval start values otherwise
 # 	center = FALSE: return start values
 #   center = NA: return centers for x/y raster, otherwise start values
 #   add_max = TRUE: add in addition to x and y start values an x_max and y_max end values
-expand_dimensions.dimensions = function(x, ..., max = FALSE, center = NA) {
+
 
 	if (length(center) == 1)
 		center = setNames(rep(center, length(x)), names(x))
@@ -530,15 +577,11 @@ expand_dimensions.dimensions = function(x, ..., max = FALSE, center = NA) {
 
 	dimensions = x
 	xy = attr(x, "raster")$dimensions
-	gt = get_geotransform(x)
+	gt = st_geotransform(x)
 	lst = vector("list", length(dimensions))
 	names(lst) = names(dimensions)
 	if (! is.null(xy) && all(!is.na(xy))) { # we have raster: where defaulting to 0.5
 		where[xy] = ifelse(!max[xy] & (is.na(center[xy]) | center[xy]), 0.5, where[xy])
-#		where_xy = if (!max && (is.na(center) || center))
-#				0.5
-#			else
-#				where
 		if (xy[1] %in% names(lst)) # x
 			lst[[ xy[1] ]] = get_dimension_values(dimensions[[ xy[1] ]], where[[ xy[1] ]], gt, "x")
 		if (xy[2] %in% names(lst))  # y
@@ -563,45 +606,66 @@ dim.dimensions = function(x) {
 		lengths(expand_dimensions(x)) # FIXME: optimise?
 }
 
+#' @name print_stars
+#' @param digits number of digits to print numbers
+#' @param usetz logical; used to format \code{PCICt} or \code{POSIXct} values
+#' @param stars_crs maximum width of string for CRS objects
+#' @param all logical; if \code{TRUE} print also fields entirely filled with \code{NA} or \code{NULL}
 #' @export
-as.data.frame.dimensions = function(x, ..., digits = 6, usetz = TRUE, stars_crs = getOption("stars.crs") %||% 28) {
-	lst = lapply(x, function(y) {
-			if (length(y$values) > 3) {
-				y$values = if (is.array(y$values))
-						paste0("[", paste(dim(y$values), collapse = "x"), "] ", 
-							format(min(y$values), digits = digits), ",...,", 
-							format(max(y$values), digits = digits))
-					else if (inherits(y$values[[1]], "crs"))
-						paste0(format(y$values[[1]]), ",...,", format(y$values[[length(y$values)]]))
-					else
-						paste0(format(head(y$values, 1)), ",...,", 
-							format(tail(y$values, 1)))
-			}
-			if (is.na(y$refsys))
-				y$refsys = NA_character_
-			else if (nchar(tail(format(y$refsys), 1)) > stars_crs)
-				y$refsys = paste0(substr(tail(format(y$refsys), 1), 1L, stars_crs - 3),"...")
-			y
-		}
-	)
+as.data.frame.dimensions = function(x, ..., digits = max(3, getOption("digits")-3), usetz = TRUE, stars_crs = getOption("stars.crs") %||% 28, all = FALSE) {
 	mformat = function(x, ..., digits) {
-		if (inherits(x, c("PCICt", "POSIXct"))) 
+		if (inherits(x, c("PCICt", "POSIXct")))
 			format(x, ..., usetz = usetz)
 		else
 			format(x, digits = digits, ...) 
 	}
+	abbrev_dim = function(y) {
+		if (length(y$values) > 3 || (inherits(y$values, "sfc") && length(y$values) > 2)) {
+			y$values = if (is.array(y$values))
+					paste0("[", paste(dim(y$values), collapse = "x"), "] ", 
+						mformat(min(y$values), digits = digits), ",...,", 
+						mformat(max(y$values), digits = digits))
+				else if (inherits(y$values[[1]], "crs"))
+					paste0(format(y$values[[1]]), ",...,", format(y$values[[length(y$values)]]))
+				else if (inherits(y$values, "sfc"))
+					paste0(format(y$values[[1]], width = stars_crs), ",...,", 
+						   format(y$values[[length(y$values)]], width = stars_crs))
+				else
+					paste0(format(head(y$values, 1)), ",...,", 
+						format(tail(y$values, 1)))
+		}
+		if (is.na(y$refsys))
+			y$refsys = NA_character_
+		else if (nchar(tail(format(y$refsys), 1)) > stars_crs)
+			y$refsys = paste0(substr(tail(format(y$refsys), 1), 1L, stars_crs - 3), "...")
+		y
+	}
+	lst = lapply(x, abbrev_dim)
 	lst = lapply(lst, function(x) sapply(x, mformat, digits = digits))
 	ret = data.frame(do.call(rbind, lst), stringsAsFactors = FALSE)
+	if (! all) { # remove fields entirely NA or NULL:
+		if (all(ret$offset == "NA"))
+			ret$offset = NULL
+		if (all(ret$delta == "NA"))
+			ret$delta = NULL
+		if (all(ret$refsys == "NA"))
+			ret$refsys = NULL
+		if (all(ret$point == "NA"))
+			ret$point = NULL
+		if (all(ret$values == "NULL"))
+			ret$values = NULL
+	}
 	r = attr(x, "raster")
 	if (! any(is.na(r$dimensions))) {
 		ret$raster = rep("", nrow(ret))
 		ret[r$dimensions[1], "raster"] = "[x]"
 		ret[r$dimensions[2], "raster"] = "[y]"
-		names(ret) = c(names(lst[[1]]), "x/y")
+		names(ret)[ncol(ret)] = "x/y"
 	}
 	ret
 }
 
+#' @name print_stars
 #' @export
 print.dimensions = function(x, ...) {
 	ret = as.data.frame(x, ...)
@@ -613,21 +677,37 @@ print.dimensions = function(x, ...) {
 identical_dimensions = function(lst, ignore_resolution = FALSE, tolerance = 0) {
 	if (length(lst) > 1) {
 		d1 = attr(lst[[1]], "dimensions")
+		crs1 = st_crs(d1)
+		st_crs(d1) = NA_crs_
 		for (i in 2:length(lst)) {
 			di = attr(lst[[i]], "dimensions")
+			if (st_crs(di) != crs1) # check semantical equivalence; https://github.com/r-spatial/stars/issues/703
+				return(FALSE)
+			st_crs(di) = NA_crs_
 			if (ignore_resolution) {
 				for (j in seq_along(d1))
 					d1[[j]]$delta = d1[[j]]$to = NA_real_
 				for (j in seq_along(di))
 					di[[j]]$delta = di[[j]]$to = NA_real_
+				attr(d1, "raster")$blocksizes = NULL
+				attr(di, "raster")$blocksizes = NULL
 			}
-			if (! isTRUE(all.equal(d1, di, tolerance = tolerance)))
+			if (! isTRUE(all.equal(d1, di, tolerance = tolerance, check.attributes = FALSE)))
 				return(FALSE)
 		}
 	}
 	TRUE
 }
 
+#' @export
+all.equal.dimensions = function(target, current, ..., ignore_blocksizes = TRUE) {
+	if (ignore_blocksizes) {
+		attr(target, "raster")$blocksizes = NULL
+		attr(current, "raster")$blocksizes = NULL
+	}
+	NextMethod()
+}
+
 combine_dimensions = function(dots, along, check_dims_identical = TRUE) {
 	dims = st_dimensions(dots[[1]])
 	if (along > length(dims)) {
@@ -664,13 +744,13 @@ seq.dimension = function(from, ..., center = FALSE) { # does what expand_dimensi
 			x$values = x$values[i]
 		if (!is.na(x$from)) {
 			rang = x$from:x$to # valid range
-			if (max(i) > -1 && min(i) < 0)
+			if (!all(is.na(i)) && max(i, na.rm = TRUE) > -1 && min(i, na.rm = TRUE) < 0)
 				stop("cannot mix positive and negative indexes")
 			if (is.logical(i))
 				i = which(i)
 			else if (all(i < 0))
 				i = setdiff(rang, abs(i)) # subtract
-			if (all(diff(i) == 1)) {
+			if (!any(is.na(i)) && all(diff(i) == 1)) {
 				if (max(i) > length(rang))
 					stop("invalid range selected")
 				sel = rang[i]
@@ -707,3 +787,52 @@ dimnames.stars = function(x) {
 	names(attr(x, "dimensions")) = value
 	x
 }
+
+#' @export
+as.POSIXct.stars = function(x, ...) {
+	d = st_dimensions(x)
+	e = expand_dimensions(d)
+	for (i in seq_along(d)) {
+		if (inherits(e[[i]], c("Date", "POSIXt", "udunits", "PCICt"))) {
+			p = try(as.POSIXct(e[[i]]), silent = TRUE)
+			if (!inherits(p, "try-error"))
+				d[[i]] = create_dimension(values = p)
+		}
+	}
+	structure(x, dimensions = d)
+}
+
+#' @export
+drop_units.dimension = function(x) {
+	du = function(y) {
+		if (inherits(y, "units"))
+			units::drop_units(y)
+		else if (inherits(y, "intervals")) {
+			y$start = du(y$start)
+			y$end = du(y$end)
+			y
+		} else
+			y
+	}
+	if (!is.null(x$offset))
+		x$offset = du(x$offset)
+	if (!is.null(x$delta))
+		x$delta  = du(x$delta)
+	if (!is.null(x$values))
+		x$values = du(x$values)
+	x
+}
+
+#' @export
+units.dimension = function(x) {
+	if (inherits(x$offset, "units"))
+		units(x$offset)
+	else if (inherits(x$delta, "units"))
+		units(x$delta)
+	else if (inherits(x$values, "units"))
+		units(x$values)
+	else if (inherits(x$values$start, "units"))
+		units(x$values$start)
+	else
+		NULL
+}
