@@ -32,7 +32,7 @@
 #' curvilinear coordinates if they are not supplied.
 #' @param eps numeric; dimension value increases are considered identical when they differ less than \code{eps}
 #' @param ignore_bounds logical; should bounds values for dimensions, if present, be ignored?
-#' @param make_time if \code{TRUE} (the default), an atttempt is made to provide a date-time class from the "time" variable
+#' @param make_time if \code{TRUE} (the default), an attempt is made to provide a date-time class from the "time" variable
 #' @param make_units if \code{TRUE} (the default), an attempt is made to set the units property of each variable
 #' @param proxy logical; if \code{TRUE}, an object of class \code{stars_proxy} is read which contains array
 #' metadata only; if \code{FALSE} the full array data is read in memory. If not set, defaults to \code{TRUE}
@@ -45,7 +45,7 @@
 #' @details
 #' If \code{var} is not set the first set of variables on a shared grid is used.
 #'
-#' \code{start} and \code{count} columns of ncsub must correspond to the variable dimemsion (nrows)
+#' \code{start} and \code{count} columns of ncsub must correspond to the variable dimension (nrows)
 #' and be valid index using \code{\link[RNetCDF]{var.get.nc}} convention (start is 1-based). If the count value
 #' is \code{NA} then all steps are included. Axis order must match that of the variable/s being read.
 #' @export
@@ -518,7 +518,7 @@ read_ncdf = function(.x, ..., var = NULL, ncsub = NULL, curvilinear = character(
                        dims$id[!dims$id %in% dim_matcher])
       dim_matcher <- unique(dim_matcher)
     }
-    dims <- dims[match(dims$id, dim_matcher), ]
+    dims <- dims[match(dim_matcher, dims$id, nomatch = 0L), ]
   }
   return(dims)
 }
@@ -708,10 +708,10 @@ read_ncdf = function(.x, ..., var = NULL, ncsub = NULL, curvilinear = character(
 			coords[[lon_coord]] <- ((coords[[lon_coord]] + 180) %% 360) - 180
 		} else if(!is.null(lons) && .is_degrees(atts, lon_coord) &&
 				  max(lons) > 180) {
-			message(paste0("0-360 longitude crossing the international date", 
+			message(paste("0-360 longitude crossing the international date", 
 						   "line encountered.\nLongitude coordinates will be",
 						   "0-360 in output."))
-			
+
 		}
 	}
 	coords
