@@ -54,16 +54,17 @@ plot.stars_proxy = function(x, y, ..., downsample = get_downsample(dim(x))) {
 	plot(st_as_stars(x, downsample = downsample, ...), ..., downsample = 0)
 }
 
-st_stars_proxy = function(x, dimensions, ..., NA_value, resolutions, RasterIO = list(), file_dim = NULL) {
-	stopifnot(!missing(NA_value))
-	stopifnot(!missing(resolutions))
-	stopifnot(length(list(...)) == 0)
-	stopifnot(is.list(x))
-	stopifnot(inherits(dimensions, "dimensions"))
+st_stars_proxy = function(x, dimensions, ..., NA_value, resolutions, RasterIO = list(), file_dim = NULL, class = character(0)) {
+	stopifnot(!missing(NA_value),
+		!missing(resolutions),
+		length(list(...)) == 0,
+		is.list(x),
+		inherits(dimensions, "dimensions"))
+
 	if (length(RasterIO) == 0)
 		RasterIO = NULL
 	structure(x, dimensions = dimensions, NA_value = NA_value, resolutions = resolutions,
-		RasterIO = RasterIO, file_dim = file_dim, class = c("stars_proxy", "stars"))
+		RasterIO = RasterIO, file_dim = file_dim, class = c(class, "stars_proxy", "stars"))
 }
 
 add_resolution = function(lst) {
@@ -336,7 +337,7 @@ st_as_stars.stars_proxy = function(.x, ..., downsample = 0, url = attr(.x, "url"
 		# there are cases where this is not right. Hence:
 		# TODO: only warn when there is a reason to warn.
 		if (!all(downsample == 0))
-			lapply(attr(.x, "call_list"), check_xy_warn, dimensions = st_dimensions(.x))
+			lapply(cl, check_xy_warn, dimensions = st_dimensions(.x))
 		process_call_list(fetch(.x, ..., downsample = downsample), cl, envir = envir, downsample = downsample)
 	}
 }
@@ -546,12 +547,14 @@ merge.stars_proxy = function(x, y, ..., name = "attributes") {
 
 # shrink bbox with e * width in each direction
 bb_shrink = function(bb, e) {
+	stopifnot(inherits(bb, "bbox"))
 	dx = diff(bb[c("xmin", "xmax")])
 	dy = diff(bb[c("ymin", "ymax")])
 	st_bbox(setNames(c(bb["xmin"] + e * dx, 
 		bb["ymin"] + e * dy, 
 		bb["xmax"] - e * dx, 
-		bb["ymax"] - e * dy), c("xmin", "ymin", "xmax", "ymax")))
+		bb["ymax"] - e * dy), c("xmin", "ymin", "xmax", "ymax")),
+		crs = st_crs(bb))
 }
 
 #' @name st_crop
@@ -566,10 +569,7 @@ st_crop.stars_proxy = function(x, y, ..., crop = TRUE, epsilon = sqrt(.Machine$d
 		rast = attr(dm, "raster")$dimensions
 		xd = rast[1]
 		yd = rast[2]
-		bb = if (!inherits(y, "bbox"))
-				st_bbox(y)
-			else
-				y
+		bb = st_bbox(y)
 		if (epsilon != 0)
 			bb = bb_shrink(bb, epsilon)
 		# FIXME: document how EXACTLY cropping works; https://github.com/hypertidy/tidync/issues/73
