@@ -1,13 +1,15 @@
 ## ----setup, include=FALSE-----------------------------------------------------
-knitr::opts_chunk$set(echo = TRUE)
-knitr::opts_chunk$set(collapse = TRUE)
+knitr::opts_chunk$set(echo = TRUE, collapse = TRUE, dev = "png")
+set.seed(13579)
 knitr::opts_chunk$set(fig.height = 4.5)
 knitr::opts_chunk$set(fig.width = 6)
-set.seed(13579)
-EVAL = x = suppressWarnings(require(starsdata, quietly = TRUE)) && sf::sf_extSoftVersion()["GDAL"] >= "2.2.0"
+EVAL = x = suppressWarnings(require(starsdata, quietly = TRUE))
 
 ## ----eval=FALSE---------------------------------------------------------------
-#  install.packages("starsdata", repos = "http://gis-bigdata.uni-muenster.de", type = "source")
+# install.packages("starsdata", repos = "http://gis-bigdata.uni-muenster.de/pebesma", type = "source")
+# # possibly after: options(timeout = 100)
+# # or from an alternative repository:
+# # install.packages("starsdata", repos = "http://pebesma.staff.ifgi.de", type = "source")
 
 ## -----------------------------------------------------------------------------
 library(stars)
@@ -38,7 +40,7 @@ x = read_stars(tif, RasterIO = rasterio)
 dim(x)
 plot(x)
 
-## ---- eval=EVAL---------------------------------------------------------------
+## ----eval=EVAL----------------------------------------------------------------
 granule = system.file("sentinel/S2A_MSIL1C_20180220T105051_N0206_R051_T32ULE_20180221T134037.zip", package = "starsdata")
 s2 = paste0("SENTINEL2_L1C:/vsizip/", granule, "/S2A_MSIL1C_20180220T105051_N0206_R051_T32ULE_20180221T134037.SAFE/MTD_MSIL1C.xml:10m:EPSG_32632")
 (p = read_stars(s2, proxy = TRUE))
@@ -47,7 +49,7 @@ s2 = paste0("SENTINEL2_L1C:/vsizip/", granule, "/S2A_MSIL1C_20180220T105051_N020
 system.time(plot(p))
 
 ## ----eval = FALSE-------------------------------------------------------------
-#  p = read_stars(s2, proxy = FALSE)
+# p = read_stars(s2, proxy = FALSE)
 
 ## -----------------------------------------------------------------------------
 methods(class = "stars_proxy")
@@ -83,14 +85,15 @@ st_dimensions(yyy) # and dimensions not adjusted
 attr(yyy, "call_list") # the name of object in the call (y) is replaced with x:
 
 ## ----eval = FALSE-------------------------------------------------------------
-#  plot(st_apply(x, c("x", "y"), range))
+# plot(st_apply(x, c("x", "y"), range))
 
 ## ----eval=EVAL----------------------------------------------------------------
 (x = st_as_stars(yyy)) # read, adrop, subset
 
 ## ----eval=EVAL----------------------------------------------------------------
 # S2 10m: band 4: near infrared, band 1: red.
-ndvi = function(x) (x[4] - x[1])/(x[4] + x[1])
+#ndvi = function(x) (x[4] - x[1])/(x[4] + x[1])
+ndvi = function(x1, x2, x3, x4) (x4 - x1)/(x4 + x1)
 rm(x)
 (s2.ndvi = st_apply(p, c("x", "y"), ndvi))
 system.time(plot(s2.ndvi)) # read - compute ndvi - plot 
