@@ -1,6 +1,6 @@
 suppressPackageStartupMessages(library(stars))
 bcsd_obs
 L7_ETMs
-if (require(starsdata)) {
+if (require(starsdata, quietly = TRUE)) {
 	print(stars_sentinel2)
 }
