@@ -1,6 +1,6 @@
 # examples with the sf::nc polygon dataset NOT ABOUT NETCDF's, see e.g. stars.R netcdf.R
 suppressPackageStartupMessages(library(sf))
-nc = st_read(system.file("gpkg/nc.gpkg", package="sf"))
+nc = read_sf(system.file("gpkg/nc.gpkg", package="sf"))
 m = st_set_geometry(nc, NULL)
 n = as.matrix(m[c("BIR74", "SID74", "NWBIR74", "BIR79", "SID79", "NWBIR79")])
 dim(n) = c(county = 100, var = 3, year = 2)
@@ -20,7 +20,7 @@ st_apply(foo, c(1,3), frac2)
 st_apply(foo, c(1,3), frac2an)
 library(abind)
 (x = aperm(st_apply(foo, c(1,3), frac2), c(2,3,1)))
-y = aperm(st_apply(foo, c(1,3), frac2), c("sfc","year","frac2"))
+y = aperm(st_apply(foo, c(1,3), frac2), c("county","year","frac2"))
 all.equal(st_dimensions(x), st_dimensions(y))
 
 split(foo, 2)
